package com.xcompwiz.mystcraft.api.event;

import com.xcompwiz.mystcraft.api.linking.ILinkInfo;

import net.minecraft.entity.Entity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.ChunkPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.eventhandler.Cancelable;
import net.minecraftforge.fml.common.eventhandler.Event;

/**
 * Mystcraft link (teleport) events. These events are all fired server-side only. Client-side functionality should be handled by listening on the server side
 * and notifying the client via a network message. The events are sent over the Forge event bus.
 */
public abstract class LinkEvent extends Event {

	/** The world the entity is leaving. May be null. */
	public final World origin;
	/** The destination world. May be null. */
	public final World destination;
	/** The entity being linked */
	public final Entity entity;
	/** The link descriptor. You should not modify this object in any way */
	public final ILinkInfo info;

	public LinkEvent(World origin, World destination, Entity entity, ILinkInfo info) {
		this.origin = origin;
		this.destination = destination;
		this.entity = entity;
		this.info = info;
	}

	/**
	 * Cancel this event to prevent linking.
	 */
	@Cancelable
	public static class LinkEventAllow extends LinkEvent {
		public LinkEventAllow(World origin, Entity entity, ILinkInfo info) {
			super(origin, null, entity, info);
		}
	}

	/**
	 * Used to provide alternate values for the link
	 */
	public static class LinkEventAlter extends LinkEvent {
		/** Set this to alter the link (null until set) */
		public BlockPos spawn;
		/** Set this to alter the link (null until set) */
		public Float rotationYaw;

		public LinkEventAlter(World origin, World destination, Entity entity, ILinkInfo info) {
			super(origin, destination, entity, info);
		}
	}

	/**
	 * Called before the entity is linked
	 */
	public static class LinkEventStart extends LinkEvent {
		public LinkEventStart(World origin, Entity entity, ILinkInfo info) {
			super(origin, null, entity, info);
		}
	}

	/**
	 * Called when the entity leaves their current world but has not yet entered the next
	 */
	public static class LinkEventExitWorld extends LinkEvent {
		public LinkEventExitWorld(Entity entity, ILinkInfo info) {
			super(null, null, entity, info);
		}
	}

	/**
	 * Called when the entity enters the new world and their position has been approximated. The entity is not fully associated or tuned with the world yet, and
	 * players have not been sent server-side information such as weather and time.
	 */
	public static class LinkEventEnterWorld extends LinkEvent {
		public LinkEventEnterWorld(World origin, World destination, Entity entity, ILinkInfo info) {
			super(origin, destination, entity, info);
		}
	}

	/**
	 * Called once everything is finalized. The entity is fully realized in the world and has been updated on clients.
	 */
	public static class LinkEventEnd extends LinkEvent {
		public LinkEventEnd(World origin, World destination, Entity entity, ILinkInfo info) {
			super(origin, destination, entity, info);
		}
	}
}
